# frozen_string_literal: true

module Webhooks
  class Attempt < ApplicationRecord
    belongs_to :endpoint, class_name: 'Webhooks::Endpoint', foreign_key: :webhooks_endpoint_id, inverse_of: :attempts
    belongs_to :event, class_name: 'Webhooks::Event', foreign_key: :webhooks_event_id, inverse_of: :attempts

    has_one :request, class_name: 'Webhooks::Request', foreign_key: :webhooks_attempt_id, dependent: :destroy, inverse_of: :attempt
    has_one :response, through: :request

    enum state: {
      queued: 0,
      success: 1,
      error: 2,
    }

    validates :attempt,
      numericality: {
        greater_than: 0,
        less_than_or_equal_to: Rails.application.config.webhooks.attempts.limit,
      }

    before_validation :set_retry_at, on: :update, if: %i[state_changed? error?]
    after_commit :enqueue_webhook_delivery, on: :create
    after_commit :enqueue_webhook_redelivery, on: :update, if: %i[retry_at_previously_changed? retry_at?]

    def deliver
      request || create_request!
    end

    def max_attempts?
      error? && attempt >= Rails.application.config.webhooks.attempts.limit
    end

    def reattempt(manual_reattempt: false)
      return if !manual_reattempt && max_attempts?

      # If it's a manual reattempt we need to set the counter to 1 otherwise
      # we risk hitting the configured limit.
      next_attempt = if manual_reattempt
        1
      else
        attempt + 1
      end

      Webhooks::Attempt.create!(
        endpoint: endpoint,
        event: event,
        attempt: next_attempt,
        manual: manual_reattempt,
      )
    end

    private

    def set_retry_at
      return if manual?
      return unless attempt < Rails.application.config.webhooks.attempts.limit

      self.retry_at = (60**attempt).seconds.from_now
    end

    def enqueue_webhook_redelivery
      Webhooks::DeliverJob.set(wait_until: retry_at).perform_later(self)
    end

    def enqueue_webhook_delivery
      Webhooks::DeliverJob.perform_later(self)
    end
  end
end
