# frozen_string_literal: true

module Webhooks
  class Endpoint < ApplicationRecord
    URL_REGEX = URI::DEFAULT_PARSER.make_regexp(Rails.application.config.webhooks.endpoints.valid_schemas)

    validates :url, presence: true, format: { with: URL_REGEX }, uniqueness: true
    validates :secret, presence: true, uniqueness: true
    validates :state, presence: true

    validates :event_types,
      inclusion: {
        in: Rails.application.config.webhooks.events.types,
        allow_blank: true,
      }

    enum state: {
      enabled: 0,
      disabled: 1,
      error: 2,
    }

    has_secure_token :secret, length: 36

    before_validation :normalize_url
    before_validation :generate_secret, on: :create

    has_many :attempts,
      foreign_key: :webhooks_endpoint_id,
      inverse_of: :endpoint,
      dependent: :destroy

    has_many :events, through: :attempts

    scope :subscribed, -> (event_type) {
      lhs = Arel::Nodes.build_quoted(event_type)
      any_event_type = Arel::Nodes::NamedFunction.new('ANY', [arel_table[:event_types]])

      enabled.where(Arel::Nodes::Equality.new(lhs, any_event_type).or(arel_table[:event_types].eq(nil)))
    }

    def event_types=(new_event_types)
      prepared_endpoint_types = Array.wrap(new_event_types).reject(&:blank?)

      if prepared_endpoint_types.any?
        super(prepared_endpoint_types)
      else
        super(nil)
      end
    end

    def to_s
      url
    end

    private

    def normalize_url
      self.url = url&.chomp('/')
    end

    def generate_secret
      return if secret?

      loop do
        secret = self.class.generate_unique_secure_token(length: 36)
        unless self.class.exists?(secret: secret)
          self.secret = secret
          break
        end
      end
    end
  end
end
